use rocket_db_pools::{Connection, Database};
use sqlx::types::chrono;

#[derive(Database)]
#[database("dolltags")]
pub struct DollTags(sqlx::SqlitePool);
pub type DollTagsDb = Connection<DollTags>;

// Doll Profiles stuff
#[derive(Debug)]
pub struct DollProfile {
	pub id: i64,
	pub created_at: chrono::NaiveDateTime,
	pub updated_at: Option<chrono::NaiveDateTime>,

	pub name: String,
	pub pronouns: String,
	pub handler_name: String,
	pub handler_url: Option<String>,

	pub kind: Option<String>,
	pub breed: Option<String>,
	pub behaviour: Option<String>,
	pub description: Option<String>,
	pub chassis_type: Option<String>,
	pub chassis_id: Option<String>,
	pub chassis_color: Option<String>,
	pub microchipped: bool,
}

#[derive(Debug)]
pub struct CreateDollProfile {
	pub id: i64,

	pub name: String,
	pub pronouns: String,
	pub handler_name: String,
	pub handler_url: Option<String>,

	pub kind: Option<String>,
	pub breed: Option<String>,
	pub behaviour: Option<String>,
	pub description: Option<String>,
	pub chassis_type: Option<String>,
	pub chassis_id: Option<String>,
	pub chassis_color: Option<String>,
	pub microchipped: bool,
}

pub async fn get(mut db: DollTagsDb, ident: i64) -> sqlx::Result<Option<DollProfile>> {
	sqlx::query_as!(
		DollProfile,
		r#"
			select
			id, name, pronouns, handler_name, handler_url, kind, breed, behaviour, description, chassis_type, chassis_id, chassis_color,
			microchipped as "microchipped!: bool",
			created_at as "created_at!: chrono::NaiveDateTime", updated_at as "updated_at!: Option<chrono::NaiveDateTime>"
			from doll_profiles where id = ?
		"#,
		ident
	)
	.fetch_optional(&mut **db).await
}

pub async fn create(mut db: DollTagsDb, doll: CreateDollProfile) -> sqlx::Result<i64> {
	sqlx::query!(
		r#"
			insert into doll_profiles
			(id, name, pronouns, handler_name, handler_url, kind, breed, behaviour, description, chassis_type, chassis_id, chassis_color, microchipped)
			values (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
		"#,
		doll.id,
		doll.name,
		doll.pronouns,
		doll.handler_name,
		doll.handler_url,
		doll.kind,
		doll.breed,
		doll.behaviour,
		doll.description,
		doll.chassis_type,
		doll.chassis_id,
		doll.chassis_color,
		doll.microchipped,
	).execute(&mut **db).await?;

	Ok(doll.id)
}
