#[macro_use]
extern crate rocket;
use db::{DollTags, DollTagsDb};
use regex::Regex;
use rocket::form::Form;
use rocket::fs::{relative, FileServer};
use rocket_db_pools::{sqlx, Database};
use rocket_dyn_templates::{context, Template};
use sqlx::types::chrono;

pub mod db;

pub fn id_public_to_db(id: &str) -> Option<i64> {
	let id_re = Regex::new(r"^\d{6}$").unwrap();

	if id_re.is_match(id) {
		id.parse::<i64>().ok()
	} else {
		None
	}
}
pub fn id_db_to_public(id: i64) -> String {
	format!("{:0>6}", id)
}

#[get("/")]
fn index() -> Template {
	Template::render("index", context! {})
}

#[get("/register")]
fn show_register() -> Template {
	Template::render("register", context! {})
}

// TODO: Validation
#[derive(Debug, FromForm)]
struct TagForm {
	pub ident: String,

	pub name: String,
	pub pronoun_subject: String,
	pub pronoun_object: String,
	pub pronoun_possessive: String,

	pub handler_name: String,
	pub handler_link: String,

	pub kind: String,
	pub breed: String,
	pub behaviour: String,
}

#[post("/register", data = "<tag>")]
async fn handle_register(mut db: DollTagsDb, tag: Form<TagForm>) -> Template {
	println!("register: {:?}", tag);
	let pronouns = format!(
		"{}/{}/{}",
		tag.pronoun_subject, tag.pronoun_object, tag.pronoun_possessive
	);
	sqlx::query!(
		r#"
		insert into doll_profiles
		(id,
		name, pronouns, handler_name, handler_url,
		kind, breed, behaviour,
		description, chassis_type, chassis_id, chassis_color, microchipped
		)
		values
		(?,
		?, ?, ?, ?,
		?, ?, ?,
		?, ?, ?, ?, ?)"#,
		tag.ident,
		tag.name,
		pronouns,
		tag.handler_name,
		tag.handler_link,
		tag.kind,
		tag.breed,
		tag.behaviour,
		"",
		"",
		"",
		"",
		1
	)
	.execute(&mut **db)
	.await
	.expect("fuck");
	Template::render("register", context! {})
}

#[get("/profile?<ident>")]
async fn show_profile(mut db: DollTagsDb, ident: &str) -> Template {
	#[derive(Debug)]
	struct Tag {
		pub id: i64,
		pub created_at: chrono::NaiveDateTime,
	}
	let profile = sqlx::query_as!(
		Tag,
		r#"select id, created_at as "created_at!: chrono::NaiveDateTime" from doll_profiles where id = ?"#,
		423158
	)
	.fetch_one(&mut **db)
	.await
	.expect("fuck");

	println!("{:?}", profile);

	Template::render(
		"show_profile",
		context! {
			ident,
		},
	)
}

#[launch]
fn rocket() -> _ {
	rocket::build()
		.attach(Template::fairing())
		.attach(DollTags::init())
		.mount("/assets", FileServer::from(relative!("/assets")))
		.mount(
			"/",
			routes![index, show_profile, show_register, handle_register],
		)
}
